@@ -5,7 +5,6 @@
 #   fill_final_row_vec: uses kriging to fill final row of w_0/z_0
 #   create_U: uses C++ to create sparse Matrix U
 #   create_approx: creates vecchia approximation object
-#   get_row_col_pointers: gets sparse matrix indices
 #   update_obs_in_approx: updates x_ord inside approx object
 #   add_pred_to_approx: incorporates predictive locations inside vecchia approx
 #   krig_vec: prediction using vecchia
@@ -13,12 +12,11 @@
 # Random MVN ------------------------------------------------------------------
 
 rand_mvn_vec <- function(approx, theta, v, mean = rep(0, nrow(approx$x_ord)),
-                         g = eps) {
+                         g = eps, scale = 1) {
   
-  Linv <- GpGp::vecchia_Linv(covparms = c(1, sqrt(theta / v / 2), v, 0), 
-                             covfun_name = "matern_isotropic",
-                             locs = approx$x_ord, NNarray = approx$NNarray)
-  sample <- GpGp::fast_Gp_sim_Linv(Linv, approx$NNarray)
+  z <- rnorm(nrow(approx$x_ord))
+  Uraw <- create_U(approx, g, theta, v, raw_form = TRUE)
+  sample <- forward_solve_raw(Uraw, z, approx$NNarray)
   sample <- sample[approx$rev_ord_obs] + mean
   return(sample)
 }
@@ -40,18 +38,31 @@ fill_final_row_vec <- function(x, w_0, D, theta_w_0, v, m) {
 
 # Create U --------------------------------------------------------------------
 
-create_U <- function(approx, g, theta, v, sep = FALSE) {
+create_U <- function(approx, g, theta, v, sep = FALSE,
+                     raw_form = FALSE) {
   
   n <- nrow(approx$x_ord)
   if (sep) {
     U <- U_entries_sep(approx$n_cores, approx$x_ord, approx$revNN, 
-                       approx$revCond, 1, theta, g, v)
+                       1, theta, g, v)
   } else U <- U_entries(approx$n_cores, approx$x_ord, approx$revNN, 
-                        approx$revCond, 1, theta, g, v)
-  U <- c(t(U))[approx$notNA]
-  U_mat <- Matrix::sparseMatrix(i = approx$col_indices, 
-                                j = approx$row_pointers, x = U, dims = c(n, n))
-  return(U_mat)
+                        1, theta, g, v)
+  
+  if (raw_form) {
+    m <- ncol(approx$NNarray) - 1
+    U <- rev_matrix(U)
+    for (i in 1:m) {
+      zeros <- (U[i, ] == 0)
+      U[i, ] <- c(U[i, !zeros], rep(0, times = sum(zeros))) 
+    }
+    return(U)
+  } else {
+    U <- c(t(U))[approx$notNA]
+    U <- Matrix::sparseMatrix(i = approx$pointers[, 2], 
+                              j = approx$pointers[, 1], x = U, 
+                              dims = c(n, n))
+    return(U)
+  } 
 }
 
 # Create Approximation --------------------------------------------------------
@@ -65,41 +76,16 @@ create_approx <- function(x, m) {
  
   NNarray <- GpGp::find_ordered_nn(x_ord, m)
   revNN <- rev_matrix(NNarray)
-  revCond <- !is.na(revNN)
   notNA <- as.vector(t(!is.na(NNarray)))
-  pointers <- get_row_col_pointers(NNarray)
+  pointers <- row_col_pointers(NNarray)
   n_cores <- parallel::detectCores(all.tests = FALSE, logical = TRUE)
   
   out <- list(m = m, ord = ord, NNarray = NNarray, revNN = revNN, 
-              notNA = notNA, revCond = revCond, 
-              row_pointers = pointers$row_pointers, 
-              col_indices = pointers$col_indices, 
+              notNA = notNA, pointers = pointers,
               n_cores = n_cores, rev_ord_obs = rev_ord_obs, x_ord = x_ord)
   return(out)
 }
 
-# Row/Column Pointers ---------------------------------------------------------
-
-get_row_col_pointers <- function(NNarray) {
-  
-  n_entries <- sum(!is.na(NNarray))
-  row_pointers <- col_indices <- rep(NA, n_entries)
-  row_pointers[1] <- col_indices[1] <- 1
-  current <- 0
-  for (k in 1:nrow(NNarray)) {
-    inds <- rev(NNarray[k, ])
-    inds0 <- inds[!is.na(inds)]
-    n0 <- length(inds0)
-    current_cols <- (1:nrow(NNarray))[inds0]
-    if (k > 1) {
-      row_pointers[current + (1:n0)] <- as.integer(rep(k, n0))
-      col_indices[current + (1:n0)] <- as.integer(current_cols)
-    }
-    current <- current + n0
-  }
-  return(list(row_pointers = row_pointers, col_indices = col_indices))
-}
-
 # Update Observation in Approx ------------------------------------------------
 
 update_obs_in_approx <- function(approx, x_new, col_index = NULL) {
@@ -128,14 +114,11 @@ add_pred_to_approx <- function(approx, x_pred, m) {
   
   NNarray <- GpGp::find_ordered_nn(x_ord, m)
   revNN <- rev_matrix(NNarray)
-  revCond <- !is.na(revNN)
   notNA <- as.vector(t(!is.na(NNarray)))
-  pointers <- get_row_col_pointers(NNarray)
+  pointers <- row_col_pointers(NNarray)
 
   out <- list(m = m, ord = ord, NNarray = NNarray, revNN = revNN, 
-              notNA = notNA, revCond = revCond, 
-              row_pointers = pointers$row_pointers, 
-              col_indices = pointers$col_indices, 
+              notNA = notNA, pointers = pointers,
               n_cores = approx$n_cores, rev_ord_obs = approx$rev_ord_obs,
               rev_ord_pred = rev_ord_pred, observed = observed, x_ord = x_ord)
   return(out)
@@ -148,7 +131,9 @@ krig_vec <- function(y, theta, g, tau2 = 1,
                      x = NULL, x_new =  NULL, # inputs required for sigma = FALSE 
                      NNarray_pred = NULL, # optional input for sigma = FALSE
                      approx = NULL, # inputs required for sigma = TRUE
-                     sep = FALSE ) {
+                     sep = FALSE,
+                     prior_mean = rep(0, length(y)), 
+                     prior_mean_new = 0) {
   
   out <- list()
   
@@ -162,19 +147,27 @@ krig_vec <- function(y, theta, g, tau2 = 1,
     for (i in 1:n_new) {
       NN <- NNarray_pred[i, ]
       x_combined <- rbind(x[NN, , drop = FALSE], x_new[i, , drop = FALSE])
-      if (sep) {
-        K <- MaternSep(x_combined, x_combined, 1, theta, g, v)
-      } else K <- Matern(sq_dist(x_combined), 1, theta, g, v)
+      if (v == 999) {
+        if (sep) {
+          K <- Exp2Sep(x_combined, x_combined, 1, theta, g)
+        } else K <- Exp2(sq_dist(x_combined), 1, theta, g)
+      } else {
+        if (sep) {
+          K <- MaternSep(x_combined, x_combined, 1, theta, g, v)
+        } else K <- Matern(sq_dist(x_combined), 1, theta, g, v)
+      }
       L <- t(chol(K))
-      out$mean[i] <- L[m + 1, 1:m] %*% forwardsolve(L[1:m, 1:m], y[NN])
+      out$mean[i] <- L[m + 1, 1:m] %*% forwardsolve(L[1:m, 1:m], y[NN] - prior_mean[NN])
       if (s2) out$s2[i] <- tau2 * (L[m + 1, m + 1] ^ 2)
     }
+    out$mean <- out$mean + prior_mean_new
     
   } else { # lite = FALSE
     
     if (is.null(approx$observed)) # add pred to approximation if not provided
       approx <- add_pred_to_approx(approx, x_new, m)
-    yo <- y[approx$ord[approx$observed]]
+    prior_mean_ordered <- prior_mean[approx$ord[approx$observed]]
+    yo <- y[approx$ord[approx$observed]] - prior_mean_ordered
     U_mat <- create_U(approx, g, theta, v, sep = sep)
     Upp <- U_mat[!approx$observed, !approx$observed]
     Uppinv <- Matrix::solve(Upp, sparse = TRUE)
@@ -182,7 +175,7 @@ krig_vec <- function(y, theta, g, tau2 = 1,
     Uop <- U_mat[approx$observed, !approx$observed]
     UopTy <- Matrix::crossprod(Uop, yo)
     mu_ordered <- -Matrix::crossprod(Uppinv, UopTy)
-    out$mean <- mu_ordered[approx$rev_ord_pred]
+    out$mean <- prior_mean_new + mu_ordered[approx$rev_ord_pred]
     out$sigma <- as.matrix(tau2 * Winv[approx$rev_ord_pred, approx$rev_ord_pred])
     
   }
